#!/usr/bin/perl
#
# stalResizeDesktopApproximately
#

use Sys::Syslog;

my $nw = $ARGV[0] ? $ARGV[0] : 0;
my $nh = $ARGV[1] ? $ARGV[1] : 0;

openlog("stal_resize", "ndelay", LOG_USER);

my %modes = ();

my $cw = 0;
my $ch = 0;

if(open(FD, "xrandr -q 2>/dev/null |"))
{
    while(<FD>)
    {
	if($_ =~ /\s+current\s+(\d+)\s+x\s+(\d+)/)
	{
	    $cw = $1;
	    $ch = $2;
	}

	if($_ =~ /\s+(\d+)x(\d+)\s+/)
	{
	    if(not exists $modes{$1})
	    {
		$modes{$1} = [];
	    }
	    push @{ $modes{$1} }, $2;
	}
    }
    close(FD);
}

sub find_value {
    my($s1, $s2, $val, undef) = @_;

    if(0 < $s1 && $s1 < $s2)
    {
        return $s1 if $s1 == $val;
        return $s2 if $s2 == $val;

        if($s1 < $val && $val < $s2)
        {
            my $dv = ($s2 - $s1) / 2;

            return $val < $s1 + $dv + 1 ? $s1 : $s2;
        }
    }

    return 0;
}

sub find_range {
    my($val, @arr) = @_;

    my $val0 = 0;

    foreach my $val1 ( @arr )
    {
        my $res = find_value($val0, $val1, $val);

	if(0 < $res)
	{
            return $res;
        }

        $val0 = $val1;
    }

    return length(@arr) == 1 ? $arr[0] : 0;
}

my $ret = 0;

if($cw != $nw || $ch != $nh)
{
    my $mw = find_range($nw, sort {$a <=> $b} keys %modes);
    my $mh = find_range($nh, sort {$a <=> $b} @{ $modes{$mw} });

    if(0 < $mw && 0 < $mh)
    {
        if($cw == $mw && $ch == $mh)
        {
            $ret = 1;
        }
        else
        {
            syslog(LOG_INFO, "cut mode: $cw $ch, req mode: $nw $nh, set mode: $mw, $mh, display $ENV{DISPLAY}");
            system("xrandr --output default --mode " . $mw . "x" . $mh . " 2>/dev/null");
        }
    }
    else
    {
        syslog(LOG_ERR, "cut mode: $cw $ch, req mode: $nw $nh, failed mode: $mw, $mh, display $ENV{DISPLAY}");
        $ret = 5;
    }
}

closelog();
exit $ret;
